@@ -29,11 +29,11 @@ import six
 
 import sge
 from sge import r
-from sge.r import (_scale, _get_blend_flags, _set_mode, _handle_music,
-                   _get_dot_sprite, _get_line_sprite, _get_rectangle_sprite,
-                   _get_ellipse_sprite, _get_circle_sprite,
-                   _get_polygon_sprite, bl_update, bl_get_image, o_update,
-                   o_detect_collisions, s_get_image)
+from sge.r import (_scale, _get_blend_flags, _screen_blend, _set_mode,
+                   _handle_music, _get_dot_sprite, _get_line_sprite,
+                   _get_rectangle_sprite, _get_ellipse_sprite,
+                   _get_circle_sprite, _get_polygon_sprite, bl_update,
+                   bl_get_image, o_update, o_detect_collisions, s_get_image)
 
 
 __all__ = ['Game']
@@ -357,48 +357,30 @@ class Game(object):
                         self.event_key_press(event.key, event.char)
                         self.current_room.event_key_press(event.key,
                                                           event.char)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_key_press(event.key, event.char)
-                            else:
-                                obj.event_inactive_key_press(event.key,
-                                                             event.char)
+                        for obj in r._active_objects.copy():
+                            obj.event_key_press(event.key, event.char)
                     elif isinstance(event, sge.input.KeyRelease):
                         self.event_key_release(event.key)
                         self.current_room.event_key_release(event.key)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_key_release(event.key)
-                            else:
-                                obj.event_inactive_key_release(event.key)
+                        for obj in r._active_objects.copy():
+                            obj.event_key_release(event.key)
                     elif isinstance(event, sge.input.MouseMove):
                         self.event_mouse_move(event.x, event.y)
                         self.current_room.event_mouse_move(event.x, event.y)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_mouse_move(event.x, event.y)
-                            else:
-                                obj.event_inactive_mouse_move(event.x, event.y)
+                        for obj in r._active_objects.copy():
+                            obj.event_mouse_move(event.x, event.y)
                     elif isinstance(event, sge.input.MouseButtonPress):
                         self.event_mouse_button_press(event.button)
                         self.current_room.event_mouse_button_press(
                             event.button)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_mouse_button_press(event.button)
-                            else:
-                                obj.event_inactive_mouse_button_press(
-                                    event.button)
+                        for obj in r._active_objects.copy():
+                            obj.event_mouse_button_press(event.button)
                     elif isinstance(event, sge.input.MouseButtonRelease):
                         self.event_mouse_button_release(event.button)
                         self.current_room.event_mouse_button_release(
                             event.button)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_mouse_button_release(event.button)
-                            else:
-                                obj.event_inactive_mouse_button_release(
-                                    event.button)
+                        for obj in r._active_objects.copy():
+                            obj.event_mouse_button_release(event.button)
                     elif isinstance(event, sge.input.JoystickAxisMove):
                         self.event_joystick_axis_move(
                             event.js_name, event.js_id, event.axis,
@@ -406,15 +388,10 @@ class Game(object):
                         self.current_room.event_joystick_axis_move(
                             event.js_name, event.js_id, event.axis,
                             event.value)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_joystick_axis_move(
-                                    event.js_name, event.js_id, event.axis,
-                                    event.value)
-                            else:
-                                obj.event_inactive_joystick_axis_move(
-                                    event.js_name, event.js_id, event.axis,
-                                    event.value)
+                        for obj in r._active_objects.copy():
+                            obj.event_joystick_axis_move(
+                                event.js_name, event.js_id, event.axis,
+                                event.value)
                     elif isinstance(event, sge.input.JoystickHatMove):
                         self.event_joystick_hat_move(
                             event.js_name, event.js_id, event.hat, event.x,
@@ -422,15 +399,10 @@ class Game(object):
                         self.current_room.event_joystick_hat_move(
                             event.js_name, event.js_id, event.hat, event.x,
                             event.y)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_joystick_hat_move(
-                                    event.js_name, event.js_id, event.hat,
-                                    event.x, event.y)
-                            else:
-                                obj.event_inactive_joystick_hat_move(
-                                    event.js_name, event.js_id, event.hat,
-                                    event.x, event.y)
+                        for obj in r._active_objects.copy():
+                            obj.event_joystick_hat_move(
+                                event.js_name, event.js_id, event.hat, event.x,
+                                event.y)
                     elif isinstance(event, sge.input.JoystickTrackballMove):
                         self.event_joystick_trackball_move(
                             event.js_name, event.js_id, event.ball, event.x,
@@ -438,39 +410,26 @@ class Game(object):
                         self.current_room.event_joystick_trackball_move(
                             event.js_name, event.js_id, event.ball, event.x,
                             event.y)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_joystick_trackball_move(
-                                    event.js_name, event.js_id, event.ball,
-                                    event.x, event.y)
-                            else:
-                                obj.event_inactive_joystick_trackball_move(
-                                    event.js_name, event.js_id, event.ball,
-                                    event.x, event.y)
+                        for obj in r._active_objects.copy():
+                            obj.event_joystick_trackball_move(
+                                event.js_name, event.js_id, event.ball,
+                                event.x, event.y)
                     elif isinstance(event, sge.input.JoystickButtonPress):
                         self.event_joystick_button_press(
                             event.js_name, event.js_id, event.button)
                         self.current_room.event_joystick_button_press(
                             event.js_name, event.js_id, event.button)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_joystick_button_press(
-                                    event.js_name, event.js_id, event.button)
-                            else:
-                                obj.event_inactive_joystick_button_press(
-                                    event.js_name, event.js_id, event.button)
+                        for obj in r._active_objects.copy():
+                            obj.event_joystick_button_press(
+                                event.js_name, event.js_id, event.button)
                     elif isinstance(event, sge.input.JoystickButtonRelease):
                         self.event_joystick_button_release(
                             event.js_name, event.js_id, event.button)
                         self.current_room.event_joystick_button_release(
                             event.js_name, event.js_id, event.button)
-                        for obj in self.current_room.objects:
-                            if obj.active:
-                                obj.event_joystick_button_release(
-                                    event.js_name, event.js_id, event.button)
-                            else:
-                                obj.event_inactive_joystick_button_release(
-                                    event.js_name, event.js_id, event.button)
+                        for obj in r._active_objects.copy():
+                            obj.event_joystick_button_release(
+                                event.js_name, event.js_id, event.button)
                     elif isinstance(event, sge.input.KeyboardFocusGain):
                         self.event_gain_keyboard_focus()
                         self.current_room.event_gain_keyboard_focus()
@@ -525,28 +484,25 @@ class Game(object):
                     bl_update(layer, time_passed)
 
                 # Update objects (including mouse)
-                for obj in self.current_room.objects:
-                    if obj.active:
-                        obj.event_begin_step(real_time_passed, delta_mult)
-                        o_update(obj, time_passed, delta_mult)
-                        obj.event_step(real_time_passed, delta_mult)
-                    else:
-                        obj.event_inactive_step(real_time_passed, delta_mult)
+                for obj in r._active_objects.copy():
+                    obj.event_begin_step(real_time_passed, delta_mult)
+                    o_update(obj, time_passed, delta_mult)
+                    obj.event_step(real_time_passed, delta_mult)
 
                 if self.collision_events_enabled:
                     # Set objects' colliders
                     room = self.current_room
                     for obj in r._colliders:
                         obj.rd["colliders"] = []
-                        for area in obj.rd["collision_areas"]:
+                        for area in obj.rd["object_areas"]:
                             if area is not None:
                                 i, j = area
-                                room_area = room.rd["collision_areas"][i][j]
+                                room_area = room.object_areas[i][j]
                             else:
-                                room_area = room.rd["collision_area_void"]
+                                room_area = room.object_area_void
 
                             for other in room_area:
-                                if (other is not obj and
+                                if (other is not obj and other.tangible and
                                         other not in obj.rd["colliders"]):
                                     obj.rd["colliders"].append(other)
 
@@ -555,12 +511,8 @@ class Game(object):
                         o_detect_collisions(obj)
 
                 # End step event
-                for obj in self.current_room.objects:
-                    if obj.active:
-                        obj.event_end_step(real_time_passed, delta_mult)
-                    else:
-                        obj.event_inactive_end_step(real_time_passed,
-                                                    delta_mult)
+                for obj in r._active_objects.copy():
+                    obj.event_end_step(real_time_passed, delta_mult)
 
                 # Set xprevious and yprevious
                 for obj in self.current_room.objects:
@@ -962,7 +914,8 @@ class Game(object):
                         images.append((img, x + view_x, y + view_y, layer.z,
                                        None))
 
-            for obj in self.current_room.objects:
+            for obj in self.current_room.get_objects_at(
+                    view.x, view.y, view.width, view.height):
                 if (obj.visible and obj.sprite is not None and
                         obj is not self.mouse):
                     img = s_get_image(obj.sprite, obj.image_index,
@@ -989,8 +942,16 @@ class Game(object):
             images.sort(key=lambda img: img[3])
 
             for img in images:
-                view_surf.blit(img[0], (img[1] - view.x, img[2] - view.y),
-                               None, _get_blend_flags(img[4]))
+                x = img[1] - view.x
+                y = img[2] - view.y
+                blend_mode = img[4]
+                if blend_mode == sge.BLEND_RGB_SCREEN:
+                    _screen_blend(view_surf, img[0], x, y, False)
+                elif blend_mode == sge.BLEND_RGBA_SCREEN:
+                    _screen_blend(view_surf, img[0], x, y, True)
+                else:
+                    flags = _get_blend_flags(blend_mode)
+                    view_surf.blit(img[0], (x, y), None, flags)
 
             r.game_display_surface.blit(
                 _scale(view_surf, view.wport, view.hport),
@@ -1002,8 +963,13 @@ class Game(object):
         self.mouse.project_cursor()
         for projection in r.game_window_projections:
             (image, x, y, blend_mode) = projection
-            flags = _get_blend_flags(blend_mode)
-            r.game_display_surface.blit(image, (x, y), None, flags)
+            if blend_mode == sge.BLEND_RGB_SCREEN:
+                _screen_blend(r.game_display_surface, image, x, y, False)
+            elif blend_mode == sge.BLEND_RGBA_SCREEN:
+                _screen_blend(r.game_display_surface, image, x, y, True)
+            else:
+                flags = _get_blend_flags(blend_mode)
+                r.game_display_surface.blit(image, (x, y), None, flags)
 
         r.game_window_projections = []
 
