@@ -31,8 +31,9 @@ import six
 
 import sge
 from sge import r
-from sge.r import (o_update_collision_lists, o_get_origin_offset, o_set_speed,
-                   s_get_image, s_get_precise_mask)
+from sge.r import (o_update_object_areas, o_update_collision_lists,
+                   o_is_other, o_get_origin_offset, o_set_speed, s_get_image,
+                   s_get_precise_mask)
 
 
 __all__ = ['Object', 'Mouse']
@@ -75,9 +76,7 @@ class Object(object):
        any other automatic functionality, such as adding
        :attr:`xvelocity` and :attr:`yvelocity`
        to :attr:`x` and :attr:`y`.  If :attr:`active` is :const:`False`,
-       automatic functionality and normal events will be disabled and
-       events which have names starting with ``event_inactive_`` will be
-       executed instead of the corresponding normal events.
+       automatic functionality and normal events will be disabled.
 
        .. note::
 
@@ -89,17 +88,6 @@ class Object(object):
           wish for the object to not perform collision events, set
           :attr:`checks_collisions` to :const:`False`.
 
-       .. note::
-
-          Making an object inactive will not likely have a significant
-          effect on performance.  For performance enhancement, it is far
-          more effective to exclude objects from collision detection.
-          Object deactivation is meant to be used to easily maintain
-          control over objects that are currently being excluded from
-          collision detection (e.g. to prevent a gravity effect that
-          would otherwise occur, or to prevent the object from moving
-          through walls).
-
     .. attribute:: checks_collisions
 
        Whether or not the object should check for collisions
@@ -354,8 +342,9 @@ class Object(object):
 
     @x.setter
     def x(self, value):
-        self.__x = value
-        o_update_collision_lists(self)
+        if self.__x != value:
+            self.__x = value
+            o_update_object_areas(self)
 
     @property
     def y(self):
@@ -363,8 +352,9 @@ class Object(object):
 
     @y.setter
     def y(self, value):
-        self.__y = value
-        o_update_collision_lists(self)
+        if self.__y != value:
+            self.__y = value
+            o_update_object_areas(self)
 
     @property
     def sprite(self):
@@ -372,10 +362,24 @@ class Object(object):
 
     @sprite.setter
     def sprite(self, value):
-        self.rd["sprite"] = value
-        if value is not None:
-            self.image_index = self.image_index % value.frames
-            o_update_collision_lists(self)
+        if self.rd["sprite"] != value:
+            self.rd["sprite"] = value
+            if value is not None:
+                self.image_index = self.image_index % value.frames
+            o_update_object_areas(self)
+
+    @property
+    def active(self):
+        return self.__active
+
+    @active.setter
+    def active(self, value):
+        if self.__active != value:
+            self.__active = value
+            if value:
+                r._active_objects.add(self)
+            else:
+                r._active_objects.discard(self)
 
     @property
     def checks_collisions(self):
@@ -383,17 +387,19 @@ class Object(object):
 
     @checks_collisions.setter
     def checks_collisions(self, value):
-        self.__checks_collisions = value
-        o_update_collision_lists(self)
+        if self.__checks_collisions != value:
+            self.__checks_collisions = value
+            o_update_collision_lists(self)
 
     @property
     def tangible(self):
-        return self.__tangible
+        return self.rd["tangible"]
 
     @tangible.setter
     def tangible(self, value):
-        self.__tangible = value
-        o_update_collision_lists(self)
+        if self.rd["tangible"] != value:
+            self.rd["tangible"] = value
+            o_update_collision_lists(self)
 
     @property
     def bbox_x(self):
@@ -401,14 +407,15 @@ class Object(object):
 
     @bbox_x.setter
     def bbox_x(self, value):
-        if value is not None:
-            self.__bbox_x = value
-        else:
-            if self.sprite is not None:
-                self.__bbox_x = self.sprite.bbox_x
+        if self.__bbox_x != value:
+            if value is not None:
+                self.__bbox_x = value
             else:
-                self.__bbox_x = 0
-        o_update_collision_lists(self)
+                if self.sprite is not None:
+                    self.__bbox_x = self.sprite.bbox_x
+                else:
+                    self.__bbox_x = 0
+            o_update_object_areas(self)
 
     @property
     def bbox_y(self):
@@ -416,14 +423,15 @@ class Object(object):
 
     @bbox_y.setter
     def bbox_y(self, value):
-        if value is not None:
-            self.__bbox_y = value
-        else:
-            if self.sprite is not None:
-                self.__bbox_y = self.sprite.bbox_y
+        if self.__bbox_y != value:
+            if value is not None:
+                self.__bbox_y = value
             else:
-                self.__bbox_y = 0
-        o_update_collision_lists(self)
+                if self.sprite is not None:
+                    self.__bbox_y = self.sprite.bbox_y
+                else:
+                    self.__bbox_y = 0
+            o_update_object_areas(self)
 
     @property
     def bbox_width(self):
@@ -431,14 +439,15 @@ class Object(object):
 
     @bbox_width.setter
     def bbox_width(self, value):
-        if value is not None:
-            self.__bbox_width = value
-        else:
-            if self.sprite is not None:
-                self.__bbox_width = self.sprite.bbox_width
+        if self.__bbox_width != value:
+            if value is not None:
+                self.__bbox_width = value
             else:
-                self.__bbox_width = 1
-        o_update_collision_lists(self)
+                if self.sprite is not None:
+                    self.__bbox_width = self.sprite.bbox_width
+                else:
+                    self.__bbox_width = 1
+            o_update_object_areas(self)
 
     @property
     def bbox_height(self):
@@ -446,14 +455,15 @@ class Object(object):
 
     @bbox_height.setter
     def bbox_height(self, value):
-        if value is not None:
-            self.__bbox_height = value
-        else:
-            if self.sprite is not None:
-                self.__bbox_height = self.sprite.bbox_height
+        if self.__bbox_height != value:
+            if value is not None:
+                self.__bbox_height = value
             else:
-                self.__bbox_height = 1
-        o_update_collision_lists(self)
+                if self.sprite is not None:
+                    self.__bbox_height = self.sprite.bbox_height
+                else:
+                    self.__bbox_height = 1
+            o_update_object_areas(self)
 
     @property
     def bbox_left(self):
@@ -493,8 +503,9 @@ class Object(object):
 
     @xvelocity.setter
     def xvelocity(self, value):
-        self.rd["xv"] = value
-        o_set_speed(self)
+        if self.rd["xv"] != value:
+            self.rd["xv"] = value
+            o_set_speed(self)
 
     @property
     def yvelocity(self):
@@ -502,8 +513,9 @@ class Object(object):
 
     @yvelocity.setter
     def yvelocity(self, value):
-        self.rd["yv"] = value
-        o_set_speed(self)
+        if self.rd["yv"] != value:
+            self.rd["yv"] = value
+            o_set_speed(self)
 
     @property
     def speed(self):
@@ -511,9 +523,10 @@ class Object(object):
 
     @speed.setter
     def speed(self, value):
-        self.rd["speed"] = value
-        self.rd["xv"] = math.cos(math.radians(self.move_direction)) * value
-        self.rd["yv"] = -math.sin(math.radians(self.move_direction)) * value
+        if self.rd["speed"] != value:
+            self.rd["speed"] = value
+            self.rd["xv"] = math.cos(math.radians(self.rd["mv_dir"])) * value
+            self.rd["yv"] = -math.sin(math.radians(self.rd["mv_dir"])) * value
 
     @property
     def move_direction(self):
@@ -521,23 +534,10 @@ class Object(object):
 
     @move_direction.setter
     def move_direction(self, value):
-        self.rd["mv_dir"] = value
-        self.rd["xv"] = math.cos(math.radians(value)) * self.speed
-        self.rd["yv"] = -math.sin(math.radians(value)) * self.speed
-
-    @property
-    def image_index(self):
-        return self.__image_index
-
-    @image_index.setter
-    def image_index(self, value):
-        if self.sprite is not None:
-            self.__image_index = value
-            while self.__image_index >= self.sprite.frames:
-                self.__image_index -= self.sprite.frames
-                self.event_animation_end()
-        else:
-            self.__image_index = 0
+        if self.rd["mv_dir"] != value:
+            self.rd["mv_dir"] = value
+            self.rd["xv"] = math.cos(math.radians(value)) * self.rd["speed"]
+            self.rd["yv"] = -math.sin(math.radians(value)) * self.rd["speed"]
 
     @property
     def image_origin_x(self):
@@ -602,7 +602,8 @@ class Object(object):
                 # from a much too high frame rate.  It would cause a
                 # division by 0 later, so this is meant to prevent that.
                 self.rd["frame_time"] = 0.000001
-                w = "Could not calculate timing for {:.2e} FPS.".format(value)
+                w = "Could not calculate timing for {:.2e} FPS.".format(
+                    value)
                 warnings.warn(w)
         else:
             self.rd["frame_time"] = None
@@ -737,20 +738,21 @@ class Object(object):
         self.__x = x
         self.__y = y
         self.z = z
-        self.active = active
+        self.__active = active
         self.__checks_collisions = checks_collisions
+        self.rd["tangible"] = tangible
         self.regulate_origin = regulate_origin
         self.collision_ellipse = collision_ellipse
         self.collision_precise = collision_precise
-        self.rd["xv"] = 0
-        self.rd["yv"] = 0
+        self.rd["xv"] = xvelocity
+        self.rd["yv"] = yvelocity
         self.rd["mv_dir"] = 0
         self.rd["speed"] = 0
         self.xacceleration = xacceleration
         self.yacceleration = yacceleration
         self.xdeceleration = xdeceleration
         self.ydeceleration = ydeceleration
-        self.__image_index = image_index
+        self.image_index = image_index
         self.image_origin_x = image_origin_x
         self.image_origin_y = image_origin_y
         self.image_xscale = image_xscale
@@ -766,7 +768,7 @@ class Object(object):
         self.rd["anim_count"] = 0
         self.__origins_x = {}
         self.__origins_y = {}
-        self.rd["collision_areas"] = []
+        self.rd["object_areas"] = set()
         self.rd["colliders"] = []
         self.__masks = {}
 
@@ -789,12 +791,11 @@ class Object(object):
         self.__bbox_height = (bbox_height if bbox_height is not None else
                               sprite_bbox_height)
 
-        self.xvelocity = xvelocity
-        self.yvelocity = yvelocity
-        self.tangible = tangible
         self.visible = visible
         self.image_fps = image_fps
 
+        o_set_speed(self)
+
     def move_x(self, move):
         """
         Move the object horizontally.  This method can be overridden to
@@ -852,46 +853,8 @@ class Object(object):
         """
         room = sge.game.current_room
         if self.tangible and self in room.objects:
-            others = []
             collisions = []
 
-            for area in self.rd["collision_areas"]:
-                if area is None:
-                    area = (None, None)
-                i, j = area
-                if (i is not None and j is not None and
-                        i < len(room.rd["collision_areas"]) and
-                        j < len(room.rd["collision_areas"][i])):
-                    room_area = room.rd["collision_areas"][i][j]
-                else:
-                    room_area = room.rd["collision_area_void"]
-
-                    if i is not None or j is not None:
-                        e = "A collision area existed in a {} object, but not in the room!".format(
-                            self.__class__.__name__)
-                        e += "\nAttempted area: ({}, {})".format(i, j)
-                        x = len(room.rd["collision_areas"])
-                        y = len(room.rd["collision_areas"][0]) if x else 0
-                        e += "\nAvailable collision areas: {}x{}".format(x, y)
-                        warnings.warn(e)
-
-                for obj in room_area:
-                    if obj is not self and obj.tangible and obj not in others:
-                        if other is None:
-                            others.append(obj)
-                        elif isinstance(other, Object):
-                            if obj is other:
-                                others.append(obj)
-                        elif isinstance(other, (list, tuple)):
-                            if obj in other:
-                                others.append(obj)
-                        else:
-                            try:
-                                if isinstance(obj, other):
-                                    others.append(obj)
-                            except TypeError:
-                                pass
-
             # Change x and y to be offset values; these are easier to use.
             if x is not None:
                 x -= self.x
@@ -903,22 +866,36 @@ class Object(object):
             else:
                 y = 0
 
-            for other in others:
-                if (self.collision_precise or self.collision_ellipse or
-                        other.collision_precise or other.collision_ellipse):
-                    # Use masks.
-                    if sge.collision.masks_collide(
-                            self.mask_x + x, self.mask_y + y, self.mask,
-                            other.mask_x, other.mask_y, other.mask):
-                        collisions.append(other)
-                else:
-                    # Use bounding boxes.
-                    if sge.collision.rectangles_collide(
-                            self.bbox_left + x, self.bbox_top + y,
-                            self.bbox_width, self.bbox_height, other.bbox_left,
-                            other.bbox_top, other.bbox_width,
-                            other.bbox_height):
-                        collisions.append(other)
+            if self.collision_precise:
+                ax = self.mask_x + x
+                ay = self.mask_y + y
+                w = len(self.mask)
+                h = len(self.mask[0]) if self.mask else 0
+            else:
+                ax = self.bbox_left + x
+                ay = self.bbox_top + y
+                w = self.bbox_width
+                h = self.bbox_height
+
+            others = room.get_objects_at(ax, ay, w, h)
+
+            for obj in others:
+                if obj is not self and obj.tangible and o_is_other(obj, other):
+                    if (self.collision_precise or self.collision_ellipse or
+                            obj.collision_precise or obj.collision_ellipse):
+                        # Use masks.
+                        if sge.collision.masks_collide(
+                                self.mask_x + x, self.mask_y + y, self.mask,
+                                obj.mask_x, obj.mask_y, obj.mask):
+                            collisions.append(obj)
+                    else:
+                        # Use bounding boxes.
+                        if sge.collision.rectangles_collide(
+                                self.bbox_left + x, self.bbox_top + y,
+                                self.bbox_width, self.bbox_height,
+                                obj.bbox_left, obj.bbox_top, obj.bbox_width,
+                                obj.bbox_height):
+                            collisions.append(obj)
 
             return collisions
         else:
@@ -963,14 +940,6 @@ class Object(object):
 
         See the documentation for :meth:`sge.Game.event_step` for more
         information.
-
-        .. note::
-
-           Automatic updates, the only occurances between this event and
-           :meth:`sge.Object.event_step`, do not occur when the object
-           is inactive, so there is no need for an "inactive" variant of
-           this event.  Use :meth:`sge.Object.event_inactive_step`
-           instead.
         """
         pass
 
@@ -1150,124 +1119,6 @@ class Object(object):
         """
         pass
 
-    def event_inactive_step(self, time_passed, delta_mult):
-        """
-        See the documentation for :meth:`sge.Object.event_step`
-        for more information.  The object is considered to be inactive
-        when :attr:`active` is :const:`False`.
-        """
-        pass
-
-    def event_inactive_end_step(self, time_passed, delta_mult):
-        """
-        See the documentation for
-        :meth:`sge.Object.event_end_step` for more information.
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-        """
-        pass
-
-    def event_inactive_key_press(self, key, char):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.KeyPress` for more
-        information.
-        """
-        pass
-
-    def event_inactive_key_release(self, key):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.KeyRelease` for more
-        information.
-        """
-        pass
-
-    def event_inactive_mouse_move(self, x, y):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.MouseMove` for more
-        information.
-        """
-        pass
-
-    def event_inactive_mouse_button_press(self, button):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.MouseButtonPress`
-        for more information.
-        """
-        pass
-
-    def event_inactive_mouse_button_release(self, button):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.MouseButtonRelease`
-        for more information.
-        """
-        pass
-
-    def event_inactive_joystick_axis_move(self, js_name, js_id, axis, value):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.JoystickAxisMove`
-        for more information.
-
-        """
-        pass
-
-    def event_inactive_joystick_hat_move(self, js_name, js_id, hat, x, y):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.JoystickHatMove` for
-        more information.
-        """
-        pass
-
-    def event_inactive_joystick_trackball_move(self, js_name, js_id, ball, x, y):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for
-        :class:`sge.input.JoystickTrackballMove` for more information.
-        """
-        pass
-
-    def event_inactive_joystick_button_press(self, js_name, js_id, button):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for :class:`sge.input.JoystickButtonPress`
-        for more information.
-        """
-        pass
-
-    def event_inactive_joystick_button_release(self, js_name, js_id, button):
-        """
-        The object is considered to be inactive when :attr:`active` is
-        :const:`False`.
-
-        See the documentation for
-        :class:`sge.input.JoystickButtonRelease` for more information.
-        """
-        pass
-
     def event_paused_step(self, time_passed, delta_mult):
         """
         See the documentation for :meth:`sge.Game.event_step` for more
@@ -1422,20 +1273,20 @@ class Mouse(Object):
     @property
     def tangible(self):
         if self.x != -1 and self.y != -1:
-            return self.__tangible
+            return self.rd["tangible"]
         else:
             return False
 
     @tangible.setter
     def tangible(self, value):
-        self.__tangible = value
-        o_update_collision_lists(self)
+        self.rd["tangible"] = value
 
     def __init__(self):
         self.__visible = True
         super(Mouse, self).__init__(0, 0, 0)
 
     def event_step(self, time_passed, delta_mult):
+        o_update_object_areas(self)
         o_update_collision_lists(self)
 
     def event_collision(self, other, xdirection, ydirection):
