@@ -28,8 +28,8 @@ import six
 
 import sge
 from sge import r
-from sge.r import (_get_blend_flags, f_split_text, s_set_size, s_refresh,
-                   s_set_transparency)
+from sge.r import (_get_blend_flags, _screen_blend, f_split_text, s_set_size,
+                   s_refresh, s_set_transparency)
 
 
 __all__ = ['Sprite']
@@ -845,9 +845,15 @@ class Sprite(object):
 
         for i in six.moves.range(self.frames):
             if frame is None or frame % self.frames == i:
-                self.rd["baseimages"][i].blit(
-                    s_set_transparency(sprite, sprite.rd["baseimages"][image]),
-                    (x, y), None, pygame_flags)
+                dsurf = self.rd["baseimages"][i]
+                ssurf = s_set_transparency(sprite,
+                                           sprite.rd["baseimages"][image])
+                if blend_mode == sge.BLEND_RGB_SCREEN:
+                    _screen_blend(dsurf, ssurf, x, y, False)
+                elif blend_mode == sge.BLEND_RGBA_SCREEN:
+                    _screen_blend(dsurf, ssurf, x, y, True)
+                else:
+                    dsurf.blit(ssurf, (x, y), None, pygame_flags)
 
         s_refresh(self)
 
