@@ -21,7 +21,7 @@ The SGE Game Engine ("SGE", pronounced like "Sage") is a general-purpose
 focus on the game itself.  This makes more rapid game development
 possible, and it also makes the SGE easy to learn.
 
-Official implementations of the SGE are `free/libre software
+The SGE is `free/libre software
 <http://gnu.org/philosophy/free-sw.html>`_, and the SGE documentation is
 free/libre as well.
 
@@ -82,11 +82,8 @@ example, if an object called ``spam`` has a Z value of ``5`` while an
 object called ``eggs`` has a Z value of ``2``, ``spam`` will obscure
 part or all of ``eggs`` when the two objects overlap.
 
-If two objects with the same Z-axis value overlap, the SGE arbitrarily
-chooses which one is considered to be closer to the viewer.  The SGE is
-allowed to change this decision, but only while the objects in question
-are not overlapping, since changing the decision while the two objects
-are overlapping would cause an undesirable flicker effect.
+If two objects with the same Z-axis value overlap, the object which was
+most recently added to the room is placed in front.
 
 The Game Loop
 -------------
@@ -303,10 +300,6 @@ only.  :meth:`sge.Sprite.draw_text`, :meth:`sge.Room.project_text`,
 and :meth:`sge.Game.project_text` support anti-aliasing in all cases.
 No other drawing or projecting methods support anti-aliasing.
 
-:data:`sge.BLEND_RGBA_SCREEN` and :data:`sge.BLEND_RGB_SCREEN` are
-unsupported. If one of these blend modes is attempted, normal blending
-will be used instead.
-
 Known Problems
 --------------
 
@@ -344,7 +337,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.19"
+__version__ = "0.20"
 
 import sys
 import os
@@ -353,7 +346,6 @@ import pygame
 
 # Constants
 IMPLEMENTATION = "Pygame SGE"
-COLLISION_AREA_SIZE_DEFAULT = 128
 
 BLEND_NORMAL = None
 BLEND_ALPHA = 1
